/*
 * 06/11/2003 - 14:55:13
 *
 * EntiteDialog.java -
 * Copyright (C) 2003 Dreux Loic
 * dreuxl@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.merise.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;

import org.analyse.core.gui.action.BasicAction;
import org.analyse.core.util.GUIUtilities;
import org.analyse.core.util.MyPanelFactory;
import org.analyse.main.Main;
import org.analyse.merise.gui.list.MeriseListModel;
import org.analyse.merise.gui.table.DictionnaireTable;
import org.analyse.merise.mcd.composant.MCDAssociation;
import org.analyse.merise.mcd.composant.MCDEntite;
import org.analyse.merise.mcd.composant.MCDObjet;

/**
 * Ce <code>JDialog</code> permet d'ajouter ou de supprimer les attributs
 * d'une entité.
 */
public class EntiteDialog extends JDialog
{
    private BasicAction ok;

    private BasicAction cancel;

    private BasicAction ajout;

    private BasicAction right, left;

    private BasicAction monter, descendre;

    private BasicAction validerNom;

    private JPanel panel;

    private JLabel title;

    private JTextField nomInfo, tailleInfo;

    private JComboBox typesInfo;

    private JTextField nom;

    private JComboBox identifiant;

    private MCDObjet mcdobject;

    private DictionnaireTable data;

    private JList droite, gauche;

    private ActionHandler handler;
    
    private MouseHandler mouseHandler;

    private MeriseListModel listGauche, listDroite;

    private String oldName;

    public EntiteDialog(DictionnaireTable dictionnaireTable)
    {
        super(Main.analyseFrame, "Entité", true);
        this.data = dictionnaireTable;

        initAction();

        listGauche = new MeriseListModel();
        listDroite = new MeriseListModel();

        Container c = this.getContentPane();
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout());

        p.add(BorderLayout.NORTH, title = MyPanelFactory.createAntialiasingTitle("Entité"));
        p.add(BorderLayout.CENTER, buildPanel());
        p.add(BorderLayout.SOUTH, MyPanelFactory.createBottomWhitePanel(new JButton(ok), new JButton(cancel)));

        gauche.setModel(listGauche);
        droite.setModel(listDroite);

        gauche.addMouseListener(mouseHandler);
        droite.addMouseListener(mouseHandler);
        
        c.add(p);

        this.pack();
        this.setResizable(false);
        GUIUtilities.centerComponent(this);
    }

    /**
     * Constuit le Panel.
     */
    private JPanel buildPanel()
    {
        panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));

        panel.add(BorderLayout.NORTH, new JPanel() {
            {
                setLayout(new BorderLayout());
                add(new JLabel("Nom : ", SwingConstants.LEFT),
                        BorderLayout.WEST);
                add(nom = new JTextField(), BorderLayout.CENTER);
                nom.setAction(validerNom);
                nom.addKeyListener(new KeyHandler());
            }
        });
        panel.add(BorderLayout.CENTER, new JPanel() {
            {
                this.setLayout(new BorderLayout());
                this.setBorder(BorderFactory.createEmptyBorder(6, 0, 0, 0));
                this.add(BorderLayout.WEST, new JScrollPane(
                        gauche = new JList()) {
                    public Dimension getMinimumSize()
                    {
                        return new Dimension(200, 200);
                    }

                    public Dimension getPreferredSize()
                    {
                        return getMinimumSize();
                    }
                });
                this.add(BorderLayout.EAST, new JPanel() {
                    {
                        this.setLayout(new BorderLayout());
                        this.add(BorderLayout.CENTER, new JScrollPane(
                                droite = new JList()));
                        JPanel toolbar = new JPanel();
                        toolbar.setLayout(new FlowLayout(FlowLayout.CENTER));
                        toolbar.add(new JButton(monter));
                        toolbar.add(new JButton(descendre));
                        this.add(BorderLayout.SOUTH, toolbar);
                    }

                    public Dimension getMinimumSize()
                    {
                        return new Dimension(200, 200);
                    }

                    public Dimension getPreferredSize()
                    {
                        return getMinimumSize();
                    }

                });
                this.add(BorderLayout.CENTER, new JPanel() {
                    {
                        this.setLayout(new FlowLayout(FlowLayout.CENTER));
                        this.add(BorderLayout.CENTER, new JPanel() {
                            {
                                this.setLayout(new BorderLayout());
                                this
                                        .add(BorderLayout.NORTH, new JButton(
                                                right));
                                this.add(BorderLayout.CENTER,
                                        new JToolBar.Separator());
                                this.add(BorderLayout.SOUTH, new JButton(left));
                            }
                        });
                    }
                });
            }
        });

        nomInfo = new JTextField();
        typesInfo = new JComboBox(data.getTypes());
        tailleInfo = new JTextField();

        return new JPanel(new BorderLayout()) {
            {
                this.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
                this.add(BorderLayout.CENTER, new JPanel(new BorderLayout()) {
                    {
                        this.setBorder(BorderFactory.createEtchedBorder());
                        this.add(BorderLayout.CENTER, panel);
                    }
                });
                this.add(BorderLayout.SOUTH, new JPanel() {
                    {
                        this.setBorder(BorderFactory.createEmptyBorder(4, 0, 0,
                                0));
                        this.setLayout(new BorderLayout());

                        this.add(BorderLayout.CENTER, new JPanel() {
                            {
                                this.setBorder(BorderFactory
                                        .createEtchedBorder());
                                this.setLayout(new BorderLayout());
                                this.add(BorderLayout.CENTER, new JPanel() {
                                    {
                                        this.setLayout(new FlowLayout());
                                        add(new JLabel("nom : "));
                                        nomInfo.setPreferredSize(new Dimension(
                                                100, 20));
                                        add(nomInfo);
                                        add(new JLabel("type : "));
                                        typesInfo
                                                .setPreferredSize(new Dimension(
                                                        100, 20));
                                        add(typesInfo);
                                        add(new JLabel("taille : "));
                                        tailleInfo
                                                .setPreferredSize(new Dimension(
                                                        50, 20));
                                        add(tailleInfo);
                                        add(new JButton(ajout));
                                    }
                                });
                            }
                        });
                    }
                });

            }
        };
    }

    /**
     * Initialise les actions.
     */
    private void initAction()
    {
        handler = new ActionHandler();
        mouseHandler = new MouseHandler();

        ok = new BasicAction("OK", "Enregistrer les changements", "OK", null,
                0, null);
        ok.addActionListener(handler);
        cancel = new BasicAction("Annuler", "Annuler les changements",
                "CANCEL", null, 0, null);
        cancel.addActionListener(handler);

        right = new BasicAction("", "Ajouter une nouvelle information",
                "RIGHT", GUIUtilities.getImageIcon("Right.png"), 0, null);
        right.addActionListener(handler);

        left = new BasicAction("", "Retirer une information existante", "LEFT",
                GUIUtilities.getImageIcon("Left.png"), 0, null);
        left.addActionListener(handler);

        monter = new BasicAction(
                "",
                "Inverser le ou les élément(s) sélectionné(s) avec le précédent",
                "UP", GUIUtilities.getImageIcon("Up.png"), 0, null);
        monter.addActionListener(handler);

        descendre = new BasicAction("",
                "Inverser le ou les élément(s) sélectionné(s) avec le suivant",
                "DOWN", GUIUtilities.getImageIcon("Down.png"), 0, null);
        descendre.addActionListener(handler);

        validerNom = new BasicAction("", "Modifier le nom de l'entité", "NAME",
                null, 0, null);
        validerNom.addActionListener(handler);

        ajout = new BasicAction("Ajout", "Ajout rapide d'une information",
                "AJOUT", null, 0, null);
        ajout.addActionListener(handler);
    }

    /**
     * Chargement d'un MCDObjet. <br>
     * Initialise le titre et affiche la fenetre.
     */
    public void load(MCDObjet mcdobject)
    {
        this.mcdobject = mcdobject;

        if (mcdobject instanceof MCDEntite)
            setTitle("Entité...");
        else if (mcdobject instanceof MCDAssociation)
            setTitle("Association...");

        title.setText("<html><font size=+1><i>" + mcdobject.getName()
                + "</i></font></html>");
        nom.setText(mcdobject.getName());

        oldName = mcdobject.getName();

        update();

        this.setVisible(true);
    }

    /**
     * Met à jour les listes.
     */
    private void update()
    {
        listGauche.clear();
        listDroite.clear();
        String code;

        for (int i = 0; i < data.getRowCount() - 1; i++) {
            code = data.getID(i);
            if (!data.getUse(i))
                listGauche.addElement(code, (String) data.getValue(code,
                        DictionnaireTable.NAME));
        }
        for (Enumeration e = mcdobject.elementsInformations(); e
                .hasMoreElements();) {
            code = (String) e.nextElement();
            listDroite.addElement(code, (String) data.getValue(code,
                    DictionnaireTable.NAME));
        }
    }

    /**
     * Ajoute une donnée dans l'entité.
     */
    private void add()
    {
        String key;
        Vector keys = new Vector();
        for (int i = 0; i < listGauche.getSize(); i++) {
            if (gauche.isSelectedIndex(i))
                keys.add(listGauche.getKey(i));
        }
        for (Enumeration e = keys.elements(); e.hasMoreElements();) {
            key = (String) e.nextElement();
            listDroite.addElement(key, (String) data.getValue(key,
                    DictionnaireTable.NAME));
            listGauche.removeElement(key);
        }
    }

    /**
     * Supprime une donnée de l'entité.
     */
    private void remove()
    {
        String key;
        Vector keys = new Vector();
        for (int i = 0; i < listDroite.getSize(); i++) {
            if (droite.isSelectedIndex(i))
                keys.add(listDroite.getKey(i));
        }
        for (Enumeration e = keys.elements(); e.hasMoreElements();) {
            key = (String) e.nextElement();
            listGauche.addElement(key, (String) data.getValue(key,
                    DictionnaireTable.NAME));
            listDroite.removeElement(key);
        }
    }

    /**
     * Déplace une donnée vers le haut de la liste.
     */
    private void up()
    {
        boolean sel = false;
        int debut = 0, fin = 0;

        for (int i = 0; i < listDroite.getSize() + 1; i++) {
            if (droite.isSelectedIndex(i) && !sel) {
                debut = i;
                sel = true;
            } else if (!droite.isSelectedIndex(i) && sel) {
                fin = i - 1;
                break;
            }
        }
        if (listDroite.moveLines(debut, fin, MeriseListModel.UP)) {
            int[] indices = new int[fin - debut + 1];
            int j = 0;
            for (int i = debut - 1; i < fin; i++) {
                indices[j] = i;
                j++;
            }

            droite.setSelectedIndices(indices);
        }
    }

    /**
     * Déplace une donnée vers le bas de la liste.
     */
    private void down()
    {
        boolean sel = false;
        int debut = 0, fin = 0;

        for (int i = 0; i < listDroite.getSize() + 1; i++) {
            if (droite.isSelectedIndex(i) && !sel) {
                debut = i;
                sel = true;
            } else if (!droite.isSelectedIndex(i) && sel) {
                fin = i - 1;
                break;
            }
        }
        if (listDroite.moveLines(debut, fin, MeriseListModel.DOWN)) {
            int[] indices = new int[fin - debut + 1];
            int j = 0;
            for (int i = debut + 1; i < fin + 2; i++) {
                indices[j] = i;
                j++;
            }

            droite.setSelectedIndices(indices);

        }
    }

    /**
     * Valide les modifications.
     */
    private void ok()
    {
        mcdobject.clearInformations();
        for (int i = 0; i < listDroite.getSize(); i++) {
            data.setUse(listDroite.getKey(i), true);
            mcdobject.addInformation(listDroite.getKey(i));
        }

        // Vérifie que le nom n'est pas utilisé par quelqu'un d'autre
        if (mcdobject.getMCD().getElement(nom.getText()) == null
                || mcdobject.getMCD().getElement(nom.getText()) == mcdobject) {
            mcdobject.setName(nom.getText());
            close();
        } else {
            GUIUtilities
                    .error((mcdobject.getMCD().getElement(nom.getText()) instanceof MCDEntite ? "L'entité \""
                            : "L'association ")
                            + nom.getText()
                            + "\" existe déjà. Veuillez changer de nom ...");
        }
    }

    /**
     * Ferme la fenetre.
     */
    private void close()
    {
        this.setVisible(false);
    }

    private class ActionHandler implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String action = e.getActionCommand();

            if (action.equals("OK")) {
                ok();
            } else if (action.equals("CANCEL")) {
                close();
            } else if (action.equals("RIGHT")) {
                add();
            } else if (action.equals("LEFT")) {
                remove();
            } else if (action.equals("UP")) {
                up();
            } else if (action.equals("DOWN")) {
                down();
            } else if (action.equals("NAME")) {
                oldName = nom.getText();
                title.setText("<html><font size=+1><b>" + nom.getText()
                        + "</b></font></html>");
            } else if (action.equals("AJOUT")) {
                data.addData(nomInfo.getText(), (String) typesInfo
                        .getSelectedItem(), tailleInfo.getText());
                update();
                nomInfo.setText("");
                tailleInfo.setText("");
                typesInfo.transferFocusBackward();
            }
        }
    }

    private class MouseHandler extends MouseAdapter
    {
        public void mouseClicked(MouseEvent e)
        {
            if(e.getClickCount() == 2)
            {
                if(e.getSource().equals(gauche))
                {
                    add();
                } else {
                    remove();
                }
            }
        }
    }
    
    private class KeyHandler implements KeyListener
    {
        public void keyPressed(KeyEvent evt)
        {
        }

        public void keyReleased(KeyEvent evt)
        {
        }

        public void keyTyped(KeyEvent evt)
        {
            // Récupère l'ancien nom dans le cas où l'utilisateur appuie sur échape
            if ((int) (evt.getKeyChar()) == 27) {
                nom.setText(oldName);
                title.setText("<html><font size=+1><b>" + nom.getText()
                        + "</b></font></html>");
            }
        }
    }
}
